@@ -26,13 +26,12 @@
 
 #include "config.h"
 
-#undef GTK_DISABLE_DEPRECATED
 #include "gtkviewport.h"
-#define GTK_DISABLE_DEPRECATED
+
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
-#include "gtktypeutils.h"
 #include "gtkscrollable.h"
+#include "gtktypebuiltins.h"
 #include "gtkprivate.h"
 
 
@@ -66,14 +65,19 @@ struct _GtkViewportPrivate
 
   GdkWindow      *bin_window;
   GdkWindow      *view_window;
+
+  /* GtkScrollablePolicy needs to be checked when
+   * driving the scrollable adjustment values */
+  guint hscroll_policy : 1;
+  guint vscroll_policy : 1;
 };
 
 enum {
   PROP_0,
   PROP_HADJUSTMENT,
   PROP_VADJUSTMENT,
-  PROP_MIN_DISPLAY_WIDTH,
-  PROP_MIN_DISPLAY_HEIGHT,
+  PROP_HSCROLL_POLICY,
+  PROP_VSCROLL_POLICY,
   PROP_SHADOW_TYPE
 };
 
@@ -139,8 +143,10 @@ gtk_viewport_class_init (GtkViewportClass *class)
   container_class->add = gtk_viewport_add;
 
   /* GtkScrollable implementation */
-  g_object_class_override_property (gobject_class, PROP_HADJUSTMENT, "hadjustment");
-  g_object_class_override_property (gobject_class, PROP_VADJUSTMENT, "vadjustment");
+  g_object_class_override_property (gobject_class, PROP_HADJUSTMENT,    "hadjustment");
+  g_object_class_override_property (gobject_class, PROP_VADJUSTMENT,    "vadjustment");
+  g_object_class_override_property (gobject_class, PROP_HSCROLL_POLICY, "hscroll-policy");
+  g_object_class_override_property (gobject_class, PROP_VSCROLL_POLICY, "vscroll-policy");
 
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
@@ -172,6 +178,14 @@ gtk_viewport_set_property (GObject         *object,
     case PROP_VADJUSTMENT:
       gtk_viewport_set_vadjustment (viewport, g_value_get_object (value));
       break;
+    case PROP_HSCROLL_POLICY:
+      viewport->priv->hscroll_policy = g_value_get_enum (value);
+      gtk_widget_queue_resize (GTK_WIDGET (viewport));
+      break;
+    case PROP_VSCROLL_POLICY:
+      viewport->priv->vscroll_policy = g_value_get_enum (value);
+      gtk_widget_queue_resize (GTK_WIDGET (viewport));
+      break;
     case PROP_SHADOW_TYPE:
       gtk_viewport_set_shadow_type (viewport, g_value_get_enum (value));
       break;
@@ -198,6 +212,12 @@ gtk_viewport_get_property (GObject         *object,
     case PROP_VADJUSTMENT:
       g_value_set_object (value, priv->vadjustment);
       break;
+    case PROP_HSCROLL_POLICY:
+      g_value_set_enum (value, priv->hscroll_policy);
+      break;
+    case PROP_VSCROLL_POLICY:
+      g_value_set_enum (value, priv->vscroll_policy);
+      break;
     case PROP_SHADOW_TYPE:
       g_value_set_enum (value, priv->shadow_type);
       break;
@@ -321,23 +341,6 @@ viewport_get_view_allocation (GtkViewport   *viewport,
   view_allocation->height = MAX (1, allocation.height - view_allocation->y * 2 - border_width * 2);
 }
 
-static void
-viewport_reclamp_adjustment (GtkAdjustment *adjustment,
-			     gboolean      *value_changed)
-{
-  gdouble value = adjustment->value;
-
-  value = CLAMP (value, 0, adjustment->upper - adjustment->page_size);
-  if (value != adjustment->value)
-    {
-      adjustment->value = value;
-      if (value_changed)
-	*value_changed = TRUE;
-    }
-  else if (value_changed)
-    *value_changed = FALSE;
-}
-
 /**
  * gtk_viewport_get_hadjustment:
  * @viewport: a #GtkViewport.
@@ -389,86 +392,101 @@ gtk_viewport_get_vadjustment (GtkViewport *viewport)
 }
 
 static void
-viewport_set_hadjustment_values (GtkViewport *viewport,
-				 gboolean    *value_changed)
+viewport_set_hadjustment_values (GtkViewport *viewport)
 {
   GtkBin *bin = GTK_BIN (viewport);
   GtkAllocation view_allocation;
   GtkAdjustment *hadjustment = gtk_viewport_get_hadjustment (viewport);
   GtkWidget *child;
-  gdouble old_page_size;
-  gdouble old_upper;
-  gdouble old_value;
+  gdouble upper, value;
   
   viewport_get_view_allocation (viewport, &view_allocation);  
 
-  old_page_size = hadjustment->page_size;
-  old_upper = hadjustment->upper;
-  old_value = hadjustment->value;
-  hadjustment->page_size = view_allocation.width;
-  hadjustment->step_increment = view_allocation.width * 0.1;
-  hadjustment->page_increment = view_allocation.width * 0.9;
-  
-  hadjustment->lower = 0;
-
   child = gtk_bin_get_child (bin);
   if (child && gtk_widget_get_visible (child))
     {
-      gint minimum_width;
+      gint minimum_width, natural_width;
+      gint scroll_height;
+      
+      if (viewport->priv->vscroll_policy == GTK_SCROLL_MINIMUM)
+	gtk_widget_get_preferred_height (child, &scroll_height, NULL);
+      else
+	gtk_widget_get_preferred_height (child, NULL, &scroll_height);
 
       gtk_widget_get_preferred_width_for_height (child,
-                                                 view_allocation.height,
+                                                 MAX (view_allocation.height, scroll_height),
                                                  &minimum_width,
-                                                 NULL);
-      hadjustment->upper = MAX (minimum_width, view_allocation.width);
-    }
-  else
-    hadjustment->upper = view_allocation.width;
+                                                 &natural_width);
 
-  if (gtk_widget_get_direction (GTK_WIDGET (viewport)) == GTK_TEXT_DIR_RTL) 
-    {
-      gdouble dist = old_upper - (old_value + old_page_size);
-      hadjustment->value = hadjustment->upper - dist - hadjustment->page_size;
-      viewport_reclamp_adjustment (hadjustment, value_changed);
-      *value_changed = (old_value != hadjustment->value);
+      if (viewport->priv->hscroll_policy == GTK_SCROLL_MINIMUM)
+	upper = MAX (minimum_width, view_allocation.width);
+      else
+	upper = MAX (natural_width, view_allocation.width);
     }
   else
-    viewport_reclamp_adjustment (hadjustment, value_changed);
+    upper = view_allocation.width;
+
+  value = gtk_adjustment_get_value (hadjustment);
+  /* We clamp to the left in RTL mode */
+  if (gtk_widget_get_direction (GTK_WIDGET (viewport)) == GTK_TEXT_DIR_RTL)
+    {
+      gdouble dist = gtk_adjustment_get_upper (hadjustment)
+                     - value
+                     - gtk_adjustment_get_page_size (hadjustment);
+      value = upper - dist - view_allocation.width;
+    }
+
+  gtk_adjustment_configure (hadjustment,
+                            value,
+                            0,
+                            upper,
+                            view_allocation.width * 0.1,
+                            view_allocation.width * 0.9,
+                            view_allocation.width);
 }
 
 static void
-viewport_set_vadjustment_values (GtkViewport *viewport,
-				 gboolean    *value_changed)
+viewport_set_vadjustment_values (GtkViewport *viewport)
 {
   GtkBin *bin = GTK_BIN (viewport);
   GtkAllocation view_allocation;
   GtkAdjustment *vadjustment = gtk_viewport_get_vadjustment (viewport);
   GtkWidget *child;
+  gdouble upper;
 
   viewport_get_view_allocation (viewport, &view_allocation);  
 
-  vadjustment->page_size = view_allocation.height;
-  vadjustment->step_increment = view_allocation.height * 0.1;
-  vadjustment->page_increment = view_allocation.height * 0.9;
-  
-  vadjustment->lower = 0;
-
   child = gtk_bin_get_child (bin);
   if (child && gtk_widget_get_visible (child))
     {
-      gint minimum_height;
+      gint minimum_height, natural_height;
+      gint scroll_width;
+
+      if (viewport->priv->hscroll_policy == GTK_SCROLL_MINIMUM)
+	gtk_widget_get_preferred_width (child, &scroll_width, NULL);
+      else
+	gtk_widget_get_preferred_width (child, NULL, &scroll_width);
 
       gtk_widget_get_preferred_height_for_width (child,
-                                                 view_allocation.width,
+                                                 MAX (view_allocation.width, scroll_width),
                                                  &minimum_height,
-                                                 NULL);
+                                                 &natural_height);
 
-      vadjustment->upper = MAX (minimum_height, view_allocation.height);
+      if (viewport->priv->vscroll_policy == GTK_SCROLL_MINIMUM)
+	upper = MAX (minimum_height, view_allocation.height);
+      else
+	upper = MAX (natural_height, view_allocation.height);
     }
   else
-    vadjustment->upper = view_allocation.height;
+    upper = view_allocation.height;
 
-  viewport_reclamp_adjustment (vadjustment, value_changed);
+  gtk_adjustment_configure (vadjustment,
+                            gtk_adjustment_get_value (vadjustment),
+                            0,
+                            upper,
+                            view_allocation.height * 0.1,
+                            view_allocation.height * 0.9,
+                            view_allocation.height);
 }
 
 static void
@@ -477,7 +495,6 @@ viewport_set_adjustment (GtkViewport    *viewport,
 			 GtkAdjustment  *adjustment)
 {
   GtkAdjustment **adjustmentp = ADJUSTMENT_POINTER (viewport, orientation);
-  gboolean value_changed;
 
   if (adjustment && adjustment == *adjustmentp)
     return;
@@ -489,20 +506,15 @@ viewport_set_adjustment (GtkViewport    *viewport,
   g_object_ref_sink (adjustment);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL)
-    viewport_set_hadjustment_values (viewport, &value_changed);
+    viewport_set_hadjustment_values (viewport);
   else
-    viewport_set_vadjustment_values (viewport, &value_changed);
+    viewport_set_vadjustment_values (viewport);
 
   g_signal_connect (adjustment, "value-changed",
 		    G_CALLBACK (gtk_viewport_adjustment_value_changed),
 		    viewport);
 
-  gtk_adjustment_changed (adjustment);
-  
-  if (value_changed)
-    gtk_adjustment_value_changed (adjustment);
-  else
-    gtk_viewport_adjustment_value_changed (adjustment, viewport);
+  gtk_viewport_adjustment_value_changed (adjustment, viewport);
 }
 
 /**
@@ -694,10 +706,10 @@ gtk_viewport_realize (GtkWidget *widget)
                                       &attributes, attributes_mask);
   gdk_window_set_user_data (priv->view_window, viewport);
 
-  attributes.x = - hadjustment->value;
-  attributes.y = - vadjustment->value;
-  attributes.width = hadjustment->upper;
-  attributes.height = vadjustment->upper;
+  attributes.x = - gtk_adjustment_get_value (hadjustment);
+  attributes.y = - gtk_adjustment_get_value (vadjustment);
+  attributes.width = gtk_adjustment_get_upper (hadjustment);
+  attributes.height = gtk_adjustment_get_upper (vadjustment);
   
   attributes.event_mask = event_mask;
 
@@ -808,7 +820,6 @@ gtk_viewport_size_allocate (GtkWidget     *widget,
   GtkViewportPrivate *priv = viewport->priv;
   GtkBin *bin = GTK_BIN (widget);
   guint border_width;
-  gboolean hadjustment_value_changed, vadjustment_value_changed;
   GtkAdjustment *hadjustment = gtk_viewport_get_hadjustment (viewport);
   GtkAdjustment *vadjustment = gtk_viewport_get_vadjustment (viewport);
   GtkAllocation child_allocation;
@@ -828,13 +839,16 @@ gtk_viewport_size_allocate (GtkWidget     *widget,
 
   gtk_widget_set_allocation (widget, allocation);
 
-  viewport_set_hadjustment_values (viewport, &hadjustment_value_changed);
-  viewport_set_vadjustment_values (viewport, &vadjustment_value_changed);
+  g_object_freeze_notify (G_OBJECT (hadjustment));
+  g_object_freeze_notify (G_OBJECT (vadjustment));
+
+  viewport_set_hadjustment_values (viewport);
+  viewport_set_vadjustment_values (viewport);
   
   child_allocation.x = 0;
   child_allocation.y = 0;
-  child_allocation.width = hadjustment->upper;
-  child_allocation.height = vadjustment->upper;
+  child_allocation.width = gtk_adjustment_get_upper (hadjustment);
+  child_allocation.height = gtk_adjustment_get_upper (vadjustment);
   if (gtk_widget_get_realized (widget))
     {
       GtkAllocation view_allocation;
@@ -852,8 +866,8 @@ gtk_viewport_size_allocate (GtkWidget     *widget,
 			      view_allocation.width,
 			      view_allocation.height);
       gdk_window_move_resize (priv->bin_window,
-                              - hadjustment->value,
-                              - vadjustment->value,
+                              - gtk_adjustment_get_value (hadjustment),
+                              - gtk_adjustment_get_value (vadjustment),
                               child_allocation.width,
                               child_allocation.height);
     }
@@ -862,12 +876,8 @@ gtk_viewport_size_allocate (GtkWidget     *widget,
   if (child && gtk_widget_get_visible (child))
     gtk_widget_size_allocate (child, &child_allocation);
 
-  gtk_adjustment_changed (hadjustment);
-  gtk_adjustment_changed (vadjustment);
-  if (hadjustment_value_changed)
-    gtk_adjustment_value_changed (hadjustment);
-  if (vadjustment_value_changed)
-    gtk_adjustment_value_changed (vadjustment);
+  g_object_thaw_notify (G_OBJECT (hadjustment));
+  g_object_thaw_notify (G_OBJECT (vadjustment));
 }
 
 static void
@@ -889,8 +899,8 @@ gtk_viewport_adjustment_value_changed (GtkAdjustment *adjustment,
       gint new_x, new_y;
 
       gdk_window_get_position (priv->bin_window, &old_x, &old_y);
-      new_x = - hadjustment->value;
-      new_y = - vadjustment->value;
+      new_x = - gtk_adjustment_get_value (hadjustment);
+      new_y = - gtk_adjustment_get_value (vadjustment);
 
       if (new_x != old_x || new_y != old_y)
 	{
